Package: future.apply
Version: 1.20.2
Title: Apply Function to Elements in Parallel using Futures
Depends: R (>= 3.2.0), future (>= 1.49.0)
Imports: globals, parallel, utils
Suggests: datasets, stats, tools, listenv, R.rsp, markdown
VignetteBuilder: R.rsp
Authors@R: c(person("Henrik", "Bengtsson",
                    role = c("aut", "cre", "cph"),
                    email = "henrikb@braju.com",
                    comment = c(ORCID = "0000-0002-7579-5165")),
             person("R Core Team", role = c("cph", "ctb")))
Description: Implementations of apply(), by(), eapply(), lapply(), Map(), .mapply(), mapply(), replicate(), sapply(), tapply(), and vapply() that can be resolved using any future-supported backend, e.g. parallel on the local machine or distributed on a compute cluster. These future_*apply() functions come with the same pros and cons as the corresponding base-R *apply() functions but with the additional feature of being able to be processed via the future framework <doi:10.32614/RJ-2021-048>.
License: GPL (>= 2)
LazyLoad: TRUE
URL: https://future.apply.futureverse.org,
        https://github.com/futureverse/future.apply
BugReports: https://github.com/futureverse/future.apply/issues
Language: en-US
Encoding: UTF-8
RoxygenNote: 7.3.3
NeedsCompilation: no
Packaged: 2026-02-20 04:06:25 UTC; hb
Author: Henrik Bengtsson [aut, cre, cph] (ORCID:
    <https://orcid.org/0000-0002-7579-5165>),
  R Core Team [cph, ctb]
Maintainer: Henrik Bengtsson <henrikb@braju.com>
Repository: CRAN
Date/Publication: 2026-02-20 12:00:09 UTC
Built: R 4.6.0; ; 2026-04-27 21:13:03 UTC; unix
