# Generated by roxygen2: do not edit by hand

S3method(future_by,data.frame)
S3method(future_by,default)
S3method(future_kernapply,default)
S3method(future_kernapply,ts)
export(future_.mapply)
export(future_Filter)
export(future_Map)
export(future_apply)
export(future_by)
export(future_eapply)
export(future_kernapply)
export(future_lapply)
export(future_mapply)
export(future_replicate)
export(future_sapply)
export(future_tapply)
export(future_vapply)
importFrom(future,Future)
importFrom(future,FutureError)
importFrom(future,FutureInterruptError)
importFrom(future,as.FutureGlobals)
importFrom(future,cancel)
importFrom(future,future)
importFrom(future,getGlobalsAndPackages)
importFrom(future,nbrOfWorkers)
importFrom(future,resolve)
importFrom(future,value)
importFrom(globals,findGlobals)
importFrom(globals,globalsByName)
importFrom(parallel,splitIndices)
importFrom(stats,end)
importFrom(stats,frequency)
importFrom(stats,kernapply)
importFrom(stats,ts)
importFrom(utils,capture.output)
importFrom(utils,globalVariables)
importFrom(utils,head)
importFrom(utils,packageVersion)
importFrom(utils,str)
