#! /usr/bin/env Rscript
## This runs testme test script inst/testme/test-future_lapply,globals.R
## Don't edit - it was autogenerated by inst/testme/deploy.R
future.apply:::testme("future_lapply,globals")
