## This runs 'testme' test inst/testme/test-<name>.R scripts
## Don't edit - it was autogenerated by inst/testme/deploy.R
testme <- function(name) {
  path <- system.file(package = 'future.apply', 'testme', mustWork = TRUE)
  Sys.setenv(R_TESTME_PATH = path)
  Sys.setenv(R_TESTME_PACKAGE = 'future.apply')
  Sys.setenv(R_TESTME_NAME = name)
  on.exit(Sys.unsetenv('R_TESTME_NAME'))
  source(file.path(path, 'run.R'))
}
